﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Data;
using Blog.Modelo;
using Microsoft.Practices.EnterpriseLibrary.Data;

namespace Blog.AcessoADados
{

    public class PostsDAO : DAO
    {
        public PostsDAO(string databaseName)
        {
            this._databaseName = databaseName;
        }

        public IList<Post> ObterTodos(Usuario usuario)
        {
            return ObterTodos(usuario, default(Paginacao));
        }
        public IList<Post> ObterTodos(Usuario usuario, Paginacao paginacao)
        {
            try
            {
                var posts = new List<Post>();
                var db = GetDb();
                using (var readerPost = db.ExecuteReader(CommandType.Text,
                    string.Format(
                      @"SELECT        Post.Id, Post.Corpo, Post.DataDePostagem, Post.Titulo, Post.Autor_id, Usuario.Email, Usuario.Nome
                    FROM          Post INNER JOIN
                                  Usuario ON Post.Autor_id = Usuario.Id
                    WHERE        (Post.Autor_id = {0})
                    ORDER BY Post.DataDePostagem DESC"
                      , usuario.Id)
                    ))
                {
                    while (readerPost.Read())
                    {
                        var post = GetPost(readerPost);
                        posts.Add(post);
                    }
                }

                using (var readerComentario = db.ExecuteReader(CommandType.Text,
                   string.Format(
                     @"SELECT        Comentario.Id, Comentario.Nome, Comentario.Email, Comentario.Site, Comentario.Corpo, Comentario.DataDePostagem, Comentario.Post_id
                   FROM          Comentario INNER JOIN
                                 Post ON Comentario.Post_id = Post.Id
                   WHERE         Post.Autor_id = {0}
                   ORDER BY      Comentario.DataDePostagem DESC"
                     , usuario.Id)
                   ))
                {
                    while (readerComentario.Read())
                    {
                        var comentario = GetComentario(readerComentario);
                        var postId = readerComentario.GetInt32(6);
                        var post = posts.Single(p => p.Id == postId);
                        post.Comentarios.Add(comentario);
                        comentario.Post = post;
                    }
                }
                
                if (paginacao.NumeroDaPagina == 0)
                    return posts;  

                return posts.AsQueryable()
                    .Take(paginacao.NumeroDaPagina * paginacao.QuantidadePorPagina)
                    .Skip((paginacao.NumeroDaPagina - 1) * paginacao.QuantidadePorPagina)
                    .ToList();
            }
            catch (Exception ex)
            {
                LogarErro(ex);
                return null;
            }
        }

        public Post ObterPorId(int id)
        {
            try
            {
                var db = GetDb();
                var readerPost = db.ExecuteReader(CommandType.Text,
                    string.Format(
                      @"SELECT        Post.Id, Post.Corpo, Post.DataDePostagem, Post.Titulo, Post.Autor_id, Usuario.Email, Usuario.Nome
                    FROM          Post INNER JOIN
                                  Usuario ON Post.Autor_id = Usuario.Id
                    WHERE        (Post.Id = {0})
                    ORDER BY Post.DataDePostagem DESC"
                      , id)
                    );
                Post post = null;
                while (readerPost.Read())
                {
                    post = GetPost(readerPost);
                }
                readerPost.Close();
                if (post == null) return null;

                ObterComentariosDoPost(db, post);
                return post;
            }
            catch (Exception ex)
            {
                LogarErro(ex);
                return null;
            }
        }

        public void Excluir(int id)
        {
            try
            {
                var db = GetDb();
                using (var trans = new System.Transactions.TransactionScope())
                {
                    db.ExecuteNonQuery(CommandType.Text,
                                       string.Format(
                                           @"DELETE FROM Comentario
                                     WHERE Post_Id = {0}",
                                           id));
                    db.ExecuteNonQuery(CommandType.Text,
                                       string.Format(
                                           @"DELETE FROM POST
                                     WHERE Id = {0}",
                                           id));
                    trans.Complete();
                }
            }
            catch (Exception ex)
            {
                LogarErro(ex);
                throw;
            }
           
        }

        public void SalvarComentario(Comentario comentario)
        {
            try
            {
                var db = GetDb();

                var total = (int)db.ExecuteScalar(CommandType.Text,
                                                   @"SELECT Count(Comentario.Id)
                                                 FROM          Comentario");
                var id = total + 1;

                var command = db.GetSqlStringCommand(
                       @"INSERT INTO Comentario (Id, Nome, Email, Site, Corpo, DataDePostagem, Post_id)
                    VALUES (@Id, @Nome, @Email, @Site, @Corpo, @DataDePostagem, @Post_id)");
                db.AddInParameter(command, "@Id", DbType.Int32);
                db.AddInParameter(command, "@Nome", DbType.String);
                db.AddInParameter(command, "@Email", DbType.String);
                db.AddInParameter(command, "@Site", DbType.String);
                db.AddInParameter(command, "@Corpo", DbType.String);
                db.AddInParameter(command, "@DataDePostagem", DbType.DateTime);
                db.AddInParameter(command, "@Post_id", DbType.Int32);
                command.Parameters[0].Value = id;
                command.Parameters[1].Value = comentario.Comentarista.Nome;
                command.Parameters[2].Value = comentario.Comentarista.Email;
                command.Parameters[3].Value = comentario.Comentarista.Site;
                command.Parameters[4].Value = comentario.Corpo;
                command.Parameters[5].Value = comentario.DataDePostagem;
                command.Parameters[6].Value = comentario.Post.Id;
                db.ExecuteNonQuery(command);
                comentario.Id = id;
            }
            catch (Exception ex)
            {
                LogarErro(ex);
            }
        }

        public void Salvar(Post post)
        {
            try
            {
                var db = GetDb();
                if (post.Id == 0)
                {
                    var total = (int)db.ExecuteScalar(CommandType.Text,
                                                          @"SELECT Count(Post.Id)
                                                        FROM          Post");
                    var id = total + 1;
                    var command = db.GetSqlStringCommand(
                        @"INSERT INTO POST (Id, Corpo, DataDePostagem, Titulo, Autor_id) 
                    VALUES (@Id, @Corpo, @DataDePostagem, @Titulo, @Autor_id)");
                    db.AddInParameter(command, "@Id", DbType.Int32);
                    db.AddInParameter(command, "@Corpo", DbType.String);
                    db.AddInParameter(command, "@DataDePostagem", DbType.DateTime);
                    db.AddInParameter(command, "@Titulo", DbType.String);
                    db.AddInParameter(command, "@Autor_id", DbType.Int32);
                    command.Parameters[0].Value = id;
                    command.Parameters[1].Value = post.Corpo;
                    command.Parameters[2].Value = post.DataDePostagem;
                    command.Parameters[3].Value = post.Titulo;
                    command.Parameters[4].Value = post.Autor.Id;
                    db.ExecuteNonQuery(command);

                    post.Id = id;
                }
                else
                {
                    var command = db.GetSqlStringCommand(
                       @"UPDATE POST 
                    SET Corpo = @Corpo, DataDePostagem = @DataDePostagem, Titulo = @Titulo, Autor_id = @Autor_id
                    WHERE Id = @Id");
                    db.AddInParameter(command, "@Id", DbType.Int32);
                    db.AddInParameter(command, "@Corpo", DbType.String);
                    db.AddInParameter(command, "@DataDePostagem", DbType.DateTime);
                    db.AddInParameter(command, "@Titulo", DbType.String);
                    db.AddInParameter(command, "@Autor_id", DbType.Int32);
                    command.Parameters[0].Value = post.Id;
                    command.Parameters[1].Value = post.Corpo;
                    command.Parameters[2].Value = post.DataDePostagem;
                    command.Parameters[3].Value = post.Titulo;
                    command.Parameters[4].Value = post.Autor.Id;
                    db.ExecuteNonQuery(command);
                }
            }
            catch (Exception ex)
            {
                LogarErro(ex);
            }
        }

        public Post ObterPorTitulo(string titulo)
        {
            try
            {
                //atenção: injeção de SQL: não faça isso!
                var db = GetDb();
                Post post = null;
                using (var readerPost = db.ExecuteReader(CommandType.Text,
                    string.Format(
                      @"SELECT        Post.Id, Post.Corpo, Post.DataDePostagem, Post.Titulo, Post.Autor_id, Usuario.Email, Usuario.Nome
                        FROM          Post INNER JOIN
                                      Usuario ON Post.Autor_id = Usuario.Id
                        WHERE        (Post.Titulo = '{0}')
                        ORDER BY Post.DataDePostagem DESC"
                      , titulo)
                    ))
                {
                    while (readerPost.Read())
                    {
                        post = GetPost(readerPost);
                    }
                }
                if (post == null) return null;

                ObterComentariosDoPost(db, post);
                return post;
            }
            catch (Exception ex)
            {
                LogarErro(ex);
                return null;
            }
            
        }


        #region Métodos privados
        
        private Post GetPost(IDataReader readerPost)
        {
            return new Post
            {
                Id = readerPost.GetInt32(0),
                Corpo = readerPost.GetString(1),
                DataDePostagem = readerPost.GetDateTime(2),
                Titulo = readerPost.GetString(3),
                Autor = new Usuario
                {
                    Id = readerPost.GetInt32(4),
                    Email = readerPost.GetString(5),
                    Nome = readerPost.GetString(6)
                }
            };
        }

        private Comentario GetComentario(IDataReader readerComentario)
        {
            return new Comentario
            {
                Id = readerComentario.GetInt32(0),
                Corpo = readerComentario.GetString(4),
                DataDePostagem = readerComentario.GetDateTime(5),
                Comentarista = new Comentarista()
                {
                    Email = readerComentario.GetString(2),
                    Nome = readerComentario.GetString(1),
                    Site = readerComentario.IsDBNull(3) ? null : readerComentario.GetString(3)
                }
            };
        }

        private void ObterComentariosDoPost(Database db, Post post)
        {
            var readerComentario = db.ExecuteReader(CommandType.Text,
                                                    string.Format(
                                                        @"SELECT        Comentario.Id, Comentario.Nome, Comentario.Email, Comentario.Site, Comentario.Corpo, Comentario.DataDePostagem, Comentario.Post_id
                                                          FROM          Comentario
                                                          WHERE         Comentario.Post_id = {0}"
                                                        , post.Id)
                );
            while (readerComentario.Read())
            {
                var comentario = GetComentario(readerComentario);
                post.Comentarios.Add(comentario);
                comentario.Post = post;
            }
            readerComentario.Close();
        }

        private string _databaseName;

        private Database GetDb()
        {
            return DatabaseFactory.CreateDatabase(_databaseName);
        }

        #endregion



    }
}
