using System;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;
using Blog.AcessoADados;
using Blog.Modelo;
using Blog.WebApp.Models;
using Blog.WebApp.Servicos;

namespace Blog.WebApp.Controllers
{
    [HandleError]
    public class PostsController : ControladorBase
    {
        //
        // GET: /Posts/
        public ActionResult Index()
        {
            try
            {
                var postsDAO = new PostsDAO("Posts");
                var posts = postsDAO.ObterTodos(Contexto.UsuarioLogado);
                return View(posts);
            }
            catch (Exception ex)
            {
                LogarErro(ex);
                throw;
            }


        }

        //
        // GET: /Posts/Details/5
        public ActionResult Details(int id)
        {
            try
            {
                var postsDAO = new PostsDAO("Posts");
                var post = postsDAO.ObterPorId(id);
                return View(post);
            }
            catch (Exception ex)
            {
                LogarErro(ex);
                throw;
            }
        }

        //
        // GET: /Posts/Create
        public ActionResult Create()
        {
            return View("Edit", new Post { DataDePostagem = DateTime.Now });
        }

        //
        // POST: /Posts/Create
        [AcceptVerbs(HttpVerbs.Post)]
        [ValidateInput(false)]
        public ActionResult Create(FormCollection collection)
        {
            return Edit(collection);
        }

        //
        // GET: /Posts/Edit/5
        public ActionResult Edit(int id)
        {
            try
            {
                var postsDAO = new PostsDAO("Posts");
                var post = postsDAO.ObterPorId(id);
                return View(post);
            }
            catch (Exception ex)
            {
                LogarErro(ex);
                throw;
            }
        }

        //
        // POST: /Posts/Edit/5
        [AcceptVerbs(HttpVerbs.Post)]
        [ValidateInput(false)]
        public ActionResult Edit(FormCollection collection)
        {
            try
            {
                var postsDAO = new PostsDAO("Posts");
                var id = Convert.ToInt32(collection["Id"]);
                Post post;
                if (id > 0)
                {
                    post = postsDAO.ObterPorId(id);
                }
                else
                {
                    post = new Post {Autor = Contexto.UsuarioLogado};
                }
                var sucesso = TryUpdateModel(post);
                Validar(post);

                if (!sucesso || !ModelState.IsValid)
                    return View("Edit", post);

                postsDAO.Salvar(post);
                return RedirectToAction("Index");

            }
            catch (Exception ex)
            {
                LogarErro(ex);
                throw;
            }
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(int id)
        {
            try
            {
                var postsDAO = new PostsDAO("Posts");
                postsDAO.Excluir(id);
                return RedirectToAction("Index");
            }
            catch (Exception ex)
            {
                LogarErro(ex);
                throw;
            }
        }


        [AcceptVerbs(HttpVerbs.Post)]
        [ValidateInput(false)]
        public ActionResult PostarComentario(int id, FormCollection collection)
        {
            Comentario comentario = null;
            Post post = null;
            try
            {
                var postsDAO = new PostsDAO("Posts");
                post = postsDAO.ObterPorId(id);
                comentario = new Comentario();
                var sucesso = TryUpdateModel(comentario);

                post.Comentarios.Add(comentario);
                comentario.Post = post;
                comentario.DataDePostagem = DateTime.Now;
                Validar(comentario);

                if (!sucesso || !ModelState.IsValid)
                {
                    var exibicao = new ExibirPost()
                    {
                        Comentario = comentario,
                        Post = post
                    };
                    return View("Post", exibicao);
                }
                if (comentario.Comentarista != null && !comentario.Comentarista.Site.ToLower().StartsWith("http"))
                {
                    comentario.Comentarista.Site = "http://" + comentario.Comentarista.Site;
                }

                new Notificador().AvisarDoComentario(comentario);

                postsDAO.SalvarComentario(comentario);
            }
            catch (Exception ex)
            {
                if (comentario == null) comentario = new Comentario();

                var exibicao = new ExibirPost()
                {
                    Comentario = comentario,
                    Post = post
                };

                LogarErro(ex);
                return View("Post", "Home", exibicao);
            }
            return RedirectToAction("Post", new { titulo = post.Titulo });
        }

        public ActionResult Post(string titulo)
        {
            try
            {
                var postsDAO = new PostsDAO("Posts");
                var post = postsDAO.ObterPorTitulo(titulo);
                if (post == null)
                    throw new HttpException(404, "Post not found.");
                var exibicao = new ExibirPost()
                {
                    Comentario = new Comentario(),
                    Post = post
                };
                return View(exibicao);
            }
            catch (Exception ex)
            {
                LogarErro(ex);
                throw;
            }
        }

        public ActionResult Errar()
        {
            try
            {
                throw new InvalidOperationException("Voc pediu por um erro...");
            }
            catch (Exception ex)
            {
                LogarErro(ex);
                throw;
            }
        }

        public void Validar(Post post)
        {
            if (string.IsNullOrEmpty(post.Corpo))
                ModelState.AddModelError("Corpo", "Corpo requerido.");
            if (string.IsNullOrEmpty(post.Titulo))
                ModelState.AddModelError("Titulo", "Titulo requerido.");
        }

        private void Validar(Comentario comentario)
        {
            if (string.IsNullOrEmpty(comentario.Corpo))
                ModelState.AddModelError("Corpo", "Corpo requerido.");
            if (comentario.Comentarista == null)
            {
                ModelState.AddModelError("Comentarista", "Comentarista requerido.");
            }
            else
            {
                if (string.IsNullOrEmpty(comentario.Comentarista.Email))
                    ModelState.AddModelError("Comentarista.Email", "Email requerido.");
                else
                {
                    const string regexEmail = @"[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?";
                    if (!Regex.Match(comentario.Comentarista.Email, regexEmail).Success)
                        ModelState.AddModelError("Comentarista.Email", "Email invlido.");
                }
                if (string.IsNullOrEmpty(comentario.Comentarista.Nome))
                    ModelState.AddModelError("Comentarista.Nome", "Nome requerido.");
            }
        }

    }
}
